@@ -20,22 +20,15 @@
 #ifndef XAP_UNIXGNOMEPRINTGRAPHICS_H
 #define XAP_UNIXGNOMEPRINTGRAPHICS_H
 
-#include "ut_vector.h"
 #include "gr_Graphics.h"
-#include "ut_misc.h"
 
 #include "xap_UnixPSFont.h"
 #include "xap_UnixFontManager.h"
 #include "xap_UnixFont.h"
 
-// hack
-extern "C" {
-#include <gnome.h>
 #include <libgnomeprint/gnome-print.h>
-#include <libgnomeprint/gnome-print-master.h>
-#include <libgnomeprint/gnome-font.h>
-#include <libgnomeprint/gnome-font-face.h>
-}
+#include <libgnomeprint/gnome-print-job.h>
+#include <libgnomeprint/gnome-print-paper.h>
 
 // forward declaration
 class UT_ByteBuf;
@@ -48,20 +41,20 @@ class XAP_UnixGnomePrintGraphics : public GR_Graphics
 {
  public:
 
-	XAP_UnixGnomePrintGraphics(GnomePrintMaster *gpm,
-				   const char * pageSize,
-				   XAP_UnixFontManager * fontManager,
-				   XAP_App *pApp, bool isPreview);
+	XAP_UnixGnomePrintGraphics(GnomePrintJob *gpm,
+							   bool isPreview = false);
 
-	XAP_UnixGnomePrintGraphics(GnomePrintContext *gpc,
-				   XAP_UnixFontManager * fontManager,
-				   XAP_App *pApp);
+	// for bonobo
+	XAP_UnixGnomePrintGraphics(GnomePrintContext *ctx,
+							   double inWidthDevice, double inHeightDevice);
 
 	virtual ~XAP_UnixGnomePrintGraphics();
 
+	virtual void drawGlyph(UT_uint32 Char, UT_sint32 xoff, UT_sint32 yoff);
 	virtual void drawChars(const UT_UCSChar* pChars, 
-			       int iCharOffset, int iLength,
-			       UT_sint32 xoff, UT_sint32 yoff);
+						   int iCharOffset, int iLength,
+						   UT_sint32 xoff, UT_sint32 yoff,
+						   int * pCharWidths = NULL);
 	virtual void setFont(GR_Font* pFont);
 
 	virtual UT_uint32 getFontAscent();
@@ -71,23 +64,29 @@ class XAP_UnixGnomePrintGraphics : public GR_Graphics
 	virtual UT_uint32 getFontAscent(GR_Font *);
 	virtual UT_uint32 getFontDescent(GR_Font *);
 	virtual UT_uint32 getFontHeight(GR_Font *);
-
 	virtual UT_uint32 measureUnRemappedChar(const UT_UCSChar c);
 	
+	virtual void getCoverage(UT_Vector& coverage);
+
 	virtual void setColor(const UT_RGBColor& clr);
-	virtual GR_Font* getGUIFont();
+	virtual void getColor(UT_RGBColor& clr);
 	virtual GR_Font* findFont(const char* pszFontFamily, 
-				  const char* pszFontStyle, 
-				  const char* pszFontVariant, 
-				  const char* pszFontWeight, 
-				  const char* pszFontStretch, 
-				  const char* pszFontSize);
+							  const char* pszFontStyle, 
+							  const char* pszFontVariant, 
+							  const char* pszFontWeight, 
+							  const char* pszFontStretch, 
+							  const char* pszFontSize);
 	virtual void drawLine(UT_sint32 x1, UT_sint32 y1, UT_sint32 x2, UT_sint32 y2);
 	virtual void setLineWidth(UT_sint32);
+	virtual void setLineProperties ( double inWidthPixels,
+					 JoinStyle inJoinStyle,
+					 CapStyle inCapStyle,
+					 LineStyle inLineStyle);
+
+	virtual GR_Font* getGUIFont();
 	virtual void xorLine(UT_sint32, UT_sint32, UT_sint32, UT_sint32);
 	virtual void polyLine(UT_Point * pts, UT_uint32 nPoints);
-	virtual void fillRect(UT_RGBColor& c, UT_sint32 x, UT_sint32 y, UT_sint32 w, UT_sint32 h);
-	virtual void fillRect(UT_RGBColor& c, UT_Rect &r);
+	virtual void fillRect(const UT_RGBColor& c, UT_sint32 x, UT_sint32 y, UT_sint32 w, UT_sint32 h);
 	virtual void invertRect(const UT_Rect*);
 	virtual void setClipRect(const UT_Rect*);
 	virtual void scroll(UT_sint32, UT_sint32);
@@ -97,19 +96,14 @@ class XAP_UnixGnomePrintGraphics : public GR_Graphics
 	virtual void clearArea(UT_sint32, UT_sint32, UT_sint32, UT_sint32);
 
 	virtual void drawImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest);
-	virtual void drawAnyImage (GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest, bool rgb);
-   	virtual GR_Image* createNewImage(const char* pszName, const UT_ByteBuf* pBBPNG, 
-					 UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight, 
-					 GR_Image::GRType iType);
+   	virtual GR_Image* createNewImage(const char* pszName, const UT_ByteBuf* pBBPNG, UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight, GR_Image::GRType iType);
 	
 	virtual bool queryProperties(GR_Graphics::Properties gp) const;
 	
 	virtual bool startPrint(void);
-	virtual bool startPage(const char * szPagelabel);
-	virtual bool startPage (const char *szPageLabel, 
-				UT_uint32, bool, 
-				UT_uint32, UT_uint32);
-	virtual bool endPrint();
+	virtual bool startPage(const char * szPagelabel, UT_uint32 pageNumber,
+							  bool bPortrait, UT_uint32 iWidth, UT_uint32 iHeight);
+	virtual bool endPrint(void);
 
 	virtual void setColorSpace(GR_Graphics::ColorSpace c);
 	virtual GR_Graphics::ColorSpace getColorSpace(void) const;
@@ -121,51 +115,56 @@ class XAP_UnixGnomePrintGraphics : public GR_Graphics
 	virtual UT_RGBColor *			getColor3D(GR_Color3D c);
 	virtual void fillRect(GR_Color3D c, UT_sint32 x, UT_sint32 y, UT_sint32 w, UT_sint32 h);
 	virtual void fillRect(GR_Color3D c, UT_Rect &r);
-	bool loadUnicodeData(void);
-	UT_uint32 getUnicodeForDingbats(UT_uint32 code);
-	UT_uint32 getUnicodeForSymbol(UT_uint32 code);
- private:
+	virtual void setPageSize(char* pageSizeName, UT_uint32 iwidth = 0, UT_uint32 iheight=0);
+	virtual void	  saveRectangle(UT_Rect & r, UT_uint32 iIndx) {}
+	virtual void	  restoreRectangle(UT_uint32 iIndx) {}
 
-	bool			_startDocument(void);
-	bool			_startPage(const char * szPageLabel);
-	bool			_endPage(void);
-	bool			_endDocument(void);
-	virtual UT_uint32       _getResolution(void) const;
+	virtual UT_uint32 getDeviceResolution(void) const;
+
+	static const guchar * s_map_page_size (const char * abi);
+	static GnomePrintConfig * s_setup_config (XAP_Frame * pFrame);
+
+protected:
+
+	virtual UT_uint32 _getResolution() const;
+
+private:
+
+	UT_sint32 scale_ydir (UT_sint32 in);
 
 	GnomeFont * _allocGnomeFont(PSFont* pFont);
-	
-	// if we need to rotate a page for landscape mode, this'll do it
-	void _setup_rotation (void);
+	void _drawAnyImage (GR_Image* pImg, 
+						UT_sint32 xDest, 
+						UT_sint32 yDest, bool rgb);
+	bool _startDocument(void);
+	bool _startPage(const char * szPageLabel);
+	bool _endPage(void);
+	bool _endDocument(void);
 
-	double _get_height (void);
-
-	/* ugly scaling functions */	
-	double _scale_factor_get(void);
-	double _scale_factor_get_inverse(void);
-	double _scale_x_dir(int x);
-	double _scale_y_dir(int y);
-
-	bool                 m_bIsPreview;
-	GnomePrintMaster        *m_gpm;
+	bool                     m_bIsPreview;
+	GnomePrintJob           *m_gpm;
 	GnomePrintContext       *m_gpc;
-	const GnomePaper        *m_paper;
+	double                   m_width, m_height;
 
 	// some small variables stolen from UnixPSGraphics
-	bool			m_bStartPrint;
-	bool			m_bStartPage;
-	bool			m_bNeedStroked;
+	bool			    m_bStartPrint;
+	bool			    m_bStartPage;
+	bool	     		m_bNeedStroked;
 	double		        m_dLineWidth;
 
 	GR_Graphics::ColorSpace	m_cs;
-	UT_RGBColor		m_currentColor;
-	PSFont *       m_pCurrentPSFont;
-	// temporary
-	GnomeFont *m_pCurrentFont;
-	bool                 m_bisSymbol;
-	bool                 m_bisDingbats;
-	UT_Vector            m_vecEncSymbol;
-	UT_Vector            m_vecEncDingbats;
+	UT_RGBColor		        m_currentColor;
+	GnomeFont *             m_pCurrentFont;
+	PSFont *                m_pCurrentPSFont;
+
 	XAP_UnixFontManager *	m_fm;
+
+	// not implemented, not possible
+	XAP_UnixGnomePrintGraphics ();
+	XAP_UnixGnomePrintGraphics (const XAP_UnixGnomePrintGraphics & other);
+	XAP_UnixGnomePrintGraphics& operator=(const XAP_UnixGnomePrintGraphics & other);
+	bool               m_bIsSymbol;
+	bool               m_bIsDingbat;
 };
 
 
