@@ -26,7 +26,7 @@
 
 PD_Style::PD_Style(pt_PieceTable * pPT, PT_AttrPropIndex indexAP, const char * szName) :
   m_pPT(pPT), m_indexAP(indexAP), m_szName(NULL), m_iUsed(0),
-  m_pBasedOn(NULL), m_pFollowedBy(NULL)
+  m_pBasedOn(NULL), m_pFollowedBy(NULL), m_iIsList(-1)
 {
   if (szName)
     m_szName = UT_strdup (szName);
@@ -112,6 +112,31 @@ bool PD_Style::isCharStyle(void) const
 	return false;
 }
 
+bool PD_Style::isList(void)
+{
+	if(m_iIsList == -1)
+	{
+		m_iIsList = 0;
+		UT_Vector vProp;
+		
+		getAllProperties(&vProp, 0);
+		
+		UT_ASSERT(vProp.getItemCount()%2 == 0);
+		
+		for(UT_uint32 i = 0; i < vProp.getItemCount(); i += 2)
+		{
+			const XML_Char * szProp  = (const XML_Char *) vProp.getNthItem(i);
+			if(!UT_strcmp(szProp, "list-style"))
+			{
+				m_iIsList = 1;
+				break;
+			}
+		}
+	}
+	
+    return m_iIsList != 0;
+}
+
 PD_Style * PD_Style::getBasedOn(void)
 {
 	if (m_pBasedOn)
@@ -194,12 +219,31 @@ bool PD_Style::addAttributes(const XML_Char ** pAtts)
 	if (!m_pPT->getAttrProp(m_indexAP, (const PP_AttrProp **)&pAP))
 		return false;
 	else
-		bres = pAP->setAttributes(pAtts);
+	{
+		if(pAP->areAlreadyPresent(pAtts,NULL))
+		{
+			return true;
+		}
+		PP_AttrProp * pNewAP = pAP->cloneWithReplacements(pAtts, NULL, false);
+		pNewAP->markReadOnly();
+		bres =	m_pPT->getVarSet().addIfUniqueAP(pNewAP, &m_indexAP);
+	}
 	m_pFollowedBy = NULL;
 	m_pBasedOn = NULL;
 	return bres;
 }
 
+size_t PD_Style::getAttributeCount(void) const
+{
+  	PP_AttrProp * pAP = NULL;
+	
+	if (!m_pPT->getAttrProp(m_indexAP, (const PP_AttrProp **)&pAP))
+		return 0;
+	else
+	        return pAP->getAttributeCount();
+}
+
+
 size_t PD_Style::getPropertyCount(void) const
 {
   	PP_AttrProp * pAP = NULL;
@@ -209,6 +253,19 @@ size_t PD_Style::getPropertyCount(void) const
 	else
 	        return pAP->getPropertyCount();
 }
+	
+bool PD_Style::getNthAttribute (int ndx, const XML_Char *&szName,
+			     const XML_Char *&szValue) const
+{
+  	PP_AttrProp * pAP = NULL;
+	
+	if (!m_pPT->getAttrProp(m_indexAP, (const PP_AttrProp **)&pAP))
+		return false;
+	else
+	  {
+	        return pAP->getNthAttribute(ndx, szName, szValue);
+	  }
+}
 
 bool PD_Style::getNthProperty (int ndx, const XML_Char *&szName,
 			       const XML_Char *&szValue) const
@@ -223,6 +280,45 @@ bool PD_Style::getNthProperty (int ndx, const XML_Char *&szName,
 	  }
 }
 
+/*!
+ * This method fills a vector structure with all the attributes defined
+ * in this style, including the basedon style.
+\param vProps the vector containing const XML_Char * (name,value) pairs
+*/
+
+bool PD_Style::getAllAttributes( UT_Vector * vAttribs, UT_sint32 depth)
+{
+//
+// This method will be recursively called to basedon style
+//
+	UT_uint32 count = getAttributeCount();
+	UT_uint32 i,j;
+	const XML_Char * szName = NULL;
+	const XML_Char * szValue = NULL;
+	for(i=0; i < count; i++)
+	{
+		getNthAttribute(i, szName, szValue);
+		bool bfound = false;
+//
+// Only keep the most recently defined properties
+//
+		for(j = 0; (j < vAttribs->getItemCount()) && !bfound ; j += 2)
+		{
+			bfound = (0 == strcmp(szName, (const char *) vAttribs->getNthItem(j)));
+		}
+		if(!bfound)
+		{
+			vAttribs->addItem((void *) szName);
+			vAttribs->addItem((void *) szValue);
+		}
+	}
+	if(depth <  pp_BASEDON_DEPTH_LIMIT && getBasedOn() != NULL)
+	{
+		getBasedOn()->getAllAttributes(vAttribs,depth +1);
+	}
+	return true;
+}
+
 /*!
  * This method fills a vector structure with all the properties defined
  * in this style, including the basedon style.
